<?
// This example shows how to create a new OWL model,
// populate it with data and present it using HTML

//  ACCESS THE STORE
//

// include OWLAPI
define('OWLAPI_INCLUDE_DIR','../../owlapi/');
require_once(OWLAPI_INCLUDE_DIR.'owlapi.php');

// connect to a POWL store
$config=array(
	'type'=>'MySQL',
	'host'=>'localhost',
	'db'=>'powl',
	'user'=>'powluser',
	'pass'=>'powlpass'
);
$powl=new POWLStore($config['type'],$config['host'],
			$config['db'],$config['user'],$config['pass']);

// get a new OWLModel
$modelURI='http://company.com/newModel';
$baseURI='http://company.com/newModel#';
if ($powl->modelExists($modelURI)) {
    trigger_error('Model with the same URI: "'.$modelURI.'" already exists',E_USER_ERROR);
} else
    $model=$powl->getNewModel($modelURI,$baseURI,'OWL');

//
//  POPULATE THE MODEL
//
	
// add a class to the model
$class=$model->addClass('MyClass');

// add a property to the model
$property=$model->addProperty('MyProperty');

// set the domain of the property to the class "MyClass"
$property->setDomain($class);

// set the range of the property to "xsd:string"
$property->setRange("xsd:string");

// add an instance of class "MyClass" to the model
$instance=$class->addInstance('MyInstance');

// attach a property value for "MyProperty" to "MyInstance"
$instance->setPropertyValue('MyProperty','This is the value of the property');

//
//  OUTPUT MODEL DATA
//

// list the classes, properties and instances in the model
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head><title>OWLAPI Example</title></head>
<body>

<h1>Classes</h1>
The model contains the following classes:
<ul>
<?
foreach($model->listClasses() as $class)
	echo('<li>'.$class->getLocalName().'</li>');
?>
</ul>

<h1>Properties</h1>
The model contains the following properties:
<ul>
<?
foreach($model->listProperties() as $property)
	echo('<li>'.$property->getLocalName().'</li>');
?>
</ul>

<h1>Instances</h1>
The model contains the following instances:
<ul>
<?
foreach($model->listInstances() as $instance) {
	$type=$instance->getClass();
	echo('<li>'.$instance->getLocalName().
		 ' - instance of '.$type->getLocalName().', properties:');
	// list all property values attached to the instance
	echo('<ul>');
	foreach($instance->listProperties() as $cl)
		foreach($cl as $prop)
			if($val=$instance->getPropertyValuePlain($prop))
				echo('<li>'.$prop->getLocalName().': '.$val.'</li>');
	echo('</ul></li>');
}
?>
</ul>
<h1>RDF Source</h1>
<xmp>
<? echo($model->writeRDFToString()); ?>
</xmp>
</body>
</html>