<?php

/**
 * Calls a method of a pOWL and hands array with parameters over
 * 
 * @param string $method A method to call
 * @param string $paramaeter_array Array with parameters for calling function
 * @return internaltype with result
 */
function _powlMethod($method, $parameter_array)
{
	global $powl;
	$result = call_user_func_array(array(& $powl, $method), $parameter_array);
	return prepare_result($result);
} 

/**
 * Calls a method of a model and hands parameters array over
 * 
 * @param string $model2use Modeluri to work on
 * @param string $method A method to call
 * @param string $paramaeter_array Array with parameters for calling function
 * @return internaltype with result
 */
function _modelMethod($model2use, $method, $parameter_array)
{
	$result = null;

	global $powl;

	$models = $powl -> ListModels();

	foreach($models as $model)
	{
		if ($model -> modelURI == $model2use)
		{
			$result = call_user_func_array(array(& $model, $method), $parameter_array);
			break;
		} 
	} 

	return prepare_result($result);
} 

/**
 * Calls a method of an instance and hands parameters array over
 * 
 * @param string $uri Instanceuri to work on
 * @param string $method A method to call
 * @param string $parameter_array Array with parameters for calling function
 * @return internaltype with result
 */
function _instanceMethod($uri, $method, $parameter_array)
{
	$result = null;

	global $powl;

	$models = $powl -> ListModels();

	foreach($models as $model)
	{
		$instance = $model -> getInstance($uri);

		if ($instance)
		{
			$result = call_user_func_array(array(& $instance, $method), $parameter_array);
			break;
		} 
	} 

	return prepare_result($result);
} 

/**
 * Calls a method of a property and hands parameters array over
 * 
 * @param string $uri Property-uri to work on
 * @param string $method A method to call
 * @param string $parameter_array Array with parameters for calling function
 * @return internaltype with result
 */
function _propertyMethod($uri, $method, $parameter_array)
{
	$result = null;
	global $powl;

	$models = $powl -> ListModels();

	foreach($models as $model)
	{
		$property = $model -> getProperty($uri);

		if ($property)
		{
			$result = call_user_func_array(array(& $property, $method), $parameter_array);
			break;
		} 
	} 

	return prepare_result($result);
} 

/**
 * Calls a method of a class and hands parameters array over
 * 
 * @param string $uri Class-uri to work on
 * @param string $method A method to call
 * @param string $paramaeter_array Array with parameters for calling function
 * @return internaltype with result
 */
function _classMethod($uri, $method, $parameter_array)
{
	$result = null;
	global $powl;

	$models = $powl -> ListModels();

	foreach($models as $model)
	{
		$class = $model -> getClass($uri);

		if ($class)
		{
			$result = call_user_func_array(array(& $class, $method), $parameter_array);
			break;
		} 
	} 
	return prepare_result($result);
} 

/**
 * Gives all registred pOWL-Installations here back
 * 
 * @return array with "servername"->"severUrl" data
 */
function _listInstallations() 
// old _getRegisteredPowls
{
	$class = pwlGetSysOntClass('Server');
	$result = $class -> listInstances();
	$data = array();

	foreach($result as $instance)
	$data[$instance -> getPropertyValuePlain('serverName')] = $instance -> getPropertyValuePlain('serverURL');

	return $data;
} 

/**
 * Registered pOWL-Installation here
 * 
 * @param string $name Name of the Installation
 * @param string $serverURL URI of the WebserviceServer
 * @return TRUE or error message
 */
function _registerInstallation($name, $serverURL) 
// old _registerPowls
{
	$currentInstallations = _listInstallations();

	if (!$key = array_search($serverURL, $currentInstallations))
	{ 
		// server not in current list
		$class = pwlGetSysOntClass('Server');
		$instance = $class -> addInstance($GLOBALS['_POWL']['SysOnt'] -> getUniqueResourceURI('Server'));
		$instance -> setPropertyValue('serverName', $name);
		$instance -> setPropertyValue('serverURL', $serverURL);
		return true;
	} elseif (isset($currentInstallations[$name]))
	{
		return $name . ' already used for server: ' . $currentInstallations[$name];
	} 
	else
	{
		return 'Server already in list, named ' . $key;
	} 
} 

/**
 * Testfunction zur Feststellunf der Verfgbarkeit dieses Webservices
 */
function _pingServer($x)
{
	return $x + 1;
} 

/**
 * Imports a model in local pOWL
 * 
 * @param string $modelURI is Uri of the model
 * @param string $baseuri is Baseuri of the model
 * @param string $modelSpec the model as rdfs-string
 * @return 1
 */
function _importModel($modelURI, $baseuri, $modelSpec)
{
	global $_ET;
	$m = $_ET['store'] -> getNewModel($modelURI, $baseuri);
	$_ET['store'] -> dbConn -> execute('LOCK TABLES statements WRITE, models WRITE');
	$m -> load($modelSpec, 'rdf');
	$_ET['store'] -> dbConn -> execute('UNLOCK TABLES');

	return 1;
} 

/**
 * Checks method to be a not-internal method, because internal shouldn't be displayed
 * 
 * @param string $string Name of the method
 * @return true if it's not beginning with '_', false else
 */
function IsNotInternalMethod($string)
{
	return (!ereg("^_", $string));
} 

/**
 * Creates an 'internaltype' result-array
 * 
 * @param  $result some result-object
 * @return resultobject as internaltype array
 */
function prepare_result($result)
{
	$ret = array();

	if ($result)
	{
		if (is_array($result)) // Is result an array type?
		{
			$ret['type'] = 'array';
			$ret['data'] = array();
			$Id = 0;
			foreach($result as $entry)
			{
				$ret['data'][$Id++] = prepare_result($entry);
			} 
		} elseif (is_object($result)) // or is it an object?
		{
			$classname = get_class($result);
			$methods = get_class_methods($classname);

			sort($methods, SORT_STRING);

			$methods = array_filter($methods, "IsNotInternalMethod");
			$ret['methods'] = $methods;
			$ret['type'] = $classname;

			if (('rdfsmodel' == $classname) || ('owlmodel' == $classname))
			{
				$ret['modelURI'] = $result -> modelURI;
				$ret['baseURI'] = $result -> getBaseUri();
				$ret['modeltype'] = $result -> gettype();
				if ($c = $result -> asResource -> getcomment())
					$ret['comment'] = $c -> getLabel();
			} elseif (('rdfsclass' == $classname) || ('owlclass' == $classname))
			{
				$ret['uri'] = $result -> geturi();
			} elseif ('rdfsinstance' == $classname)
			{
				$ret['uri'] = $result -> geturi();
			} elseif ('rdfsproperty' == $classname)
			{
				$ret['uri'] = $result -> geturi();
			} 
			else
			{
				$ret['type'] = 'unknown';
				$ret['type-name'] = $classname;
				$ret['data'] = $result;
			} 
		} 
		else
		{ 
			// ok, a simple type
			$ret['type'] = 'simple';
			$ret['data'] = $result;
		} 
	} 
	else
	{
		$ret['type'] = 'null';
	} 
	return $ret;
} 

/**
 * Prints data as lines of a htmltable
 * 
 * $data -> the data
 * 
 * $head -> with(1)/without(0) names for the row
 * 
 * $Id -> elementId
 */

function print_data($data, $head, $Id = 0)
{
	$out = "";

	if (!$data || !is_array($data) || $data['type'] == 'null')
	{
		$out .= print_null();
	} 
	elseif ($data['type'] == 'array')
	{
		// result is an array type
		$out .= print_head($data['data'][0]['type']);

		foreach($data['data'] as $entry)
		{
			$out .= print_data($entry, 1, $Id);
			$Id++;
		} 
	} 
	elseif (($data['type'] == 'rdfsmodel') || ('owlmodel' == $data['type']))
	{
		// it is an object
		if ($head != 1)
			$out .= print_head($data['type']);
		$out .= print_model($data, $Id);
	}
	elseif (($data['type'] == 'rdfsclass') || ('owlclass' == $data['type']))
	{
		if ($head != 1)
			$out .= print_head($data['type']);
		$out .= print_class($data, $Id);
	} 
	elseif ($data['type'] == 'rdfsinstance')
	{
		if ($head != 1)
			$out .= print_head($data['type']);
		$out .= print_instance($data, $Id);
	} 
	elseif ($data['type'] == 'rdfsproperty')
	{
		if ($head != 1)
			$out .= print_head($data['type']);
		$out .= print_property($data, $Id);
	} 
	elseif ($data['type'] == 'unknown')
	{
		if ($head != 1)
			$out .= print_head($data['type']);
		$out .= print_unknown($data, $Id);
	} 
	elseif ($data['type'] == 'simple')
	{
		// ok, a simple type
		$out .= print_simple($data);
	} 

	return $out;
} 

/**
 * Print all methods of object with given Position im interaltype-array
 * 
 * @param array $data Data as internaltype
 * @param integer $id Position of object in array
 * @return html -table-part as string with all methods
 */
function print_allmethods($data, $id)
{
	$return = "Error";

	if ($data['type'] == 'array')
	{
		$methods = $data['data'][$id]['methods'];
	} 
	else
	{
		$methods = $data['methods'];
	} 

	if ($methods)
	{
		$return = "<tr>";
		$return .= "<td align=\"right\">Function to call:</td>";
		$return .= "<td><select name='method'><option value='0'>Select method</option>";

		foreach($methods as $method)
			if (trim($method))
				$return .= "<option value='" . $method . "'>" . $method . "</option>";

		$return .= "</select></td>\n";
		$return .= "</tr>";
	} 

	return $return;
} 

/**
 * Gets a functionname for a ObjectType
 * 
 * @param string $kindOfObject Objectentype
 * @return Wenservice Functionname
 */
function get_functionName4Type($kindOfObject)
{
	if (($kindOfObject == 'rdfsmodel') || ('owlmodel' == $kindOfObject))
	{
		$functionCallName = 'modelMethod';
	} 
	elseif (($kindOfObject == 'rdfsclass') || ('owlclass' == $kindOfObject))
	{
		$functionCallName = 'classMethod';
	} 
	elseif ($kindOfObject == 'rdfsinstance')
	{
		$functionCallName = 'instanceMethod';
	} 
	elseif ($kindOfObject == 'rdfsproperty')
	{
		$functionCallName = 'propertyMethod';
	} 
	else
	{
		$functionCallName = null;
	} 
	return $functionCallName;
} 

/**
 * Gets the rownames for a type as line of a htmltable
 * 
 * @param string $type Type of an object
 * @return rownames 
 */

function print_head($type)
{
	$out = "<tr>\n";

	if (($type == 'rdfsmodel') || ('owlmodel' == $type))
	{
		$out .= "<th>ModelURI</th>\n";
		$out .= "<th>BaseURI</th>\n";
		$out .= "<th>Type</th>\n";
		$out .= "<th>Comment</th>\n";
		$out .= "<th>Action</th>\n";
	} 
	elseif (($type == 'rdfsclass') || ($type == 'owlclass'))
	{
		$out .= "<th>URI</th>\n";
		$out .= "<th>Action</th>\n";
	} 
	elseif ($type == 'rdfsinstance')
	{
		$out .= "<th>URI</th>\n";
		$out .= "<th>Action</th>\n";
	} 
	elseif ($type == 'rdfsproperty')
	{
		$out .= "<th>URI</th>\n";
		$out .= "<th>Action</th>\n";
	} 
	else
	{
		$out .= "<td>" . $type . "</td>";
	} 

	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a model as htmltableline
 * 
 * @param array $model pOWL Model as internaltype
 * @return html -table line with data
 */
function print_model($model, $Id)
{
	$out .= "<tr>\n";
	$out .= "<td>" . $model['modelURI'] . "</td>\n";
	$out .= "<td>" . $model['baseURI'] . "</td>\n";
	$out .= "<td>" . $model['modeltype'] . "</td>\n";
	$out .= "<td>" . $model['comment'] . "</td>\n";
	$out .= "<td>";
	$out .= "<a href='execute_server_data.php?id=" . $Id . "&type=rdfsmodel&data=" . urlencode($model['modelURI']) . "'>Execute</a> <br>";
	$out .= "<a href='import_server_data.php?data=" . urlencode($model['modelURI']) . "'>Import</a>";
	$out .= "</td>\n";
	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a Class as htmltableline
 * 
 * @param array $data pOWL Class as internaltype
 * @return html -table line with data
 */
function print_class($data, $Id)
{
	$out .= "<tr>\n";
	$out .= "<td>" . $data['uri'] . "</td>\n";
	$out .= "<td>";
	$out .= "<a href='execute_server_data.php?id=" . $Id . "&type=rdfsclass&data=" . urlencode($data['uri']) . "'>Execute</a>";
	$out .= "</td>\n";
	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a Instance as htmltableline
 * 
 * @param array $data pOWL Instance as internaltype
 * @return html -table line with data
 */
function print_instance($data, $Id)
{
	$out .= "<tr>\n";
	$out .= "<td>" . $data['uri'] . "</td>\n";
	$out .= "<td>";
	$out .= "<a href='execute_server_data.php?id=" . $Id . "&type=rdfsinstance&data=" . urlencode($data['uri']) . "'>Execute</a>";
	$out .= "</td>\n";
	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a Property as htmltableline
 * 
 * @param array $data pOWL Property as internaltype
 * @return html -table line with data
 */

function print_property($data, $Id)
{
	$out .= "<tr>\n";
	$out .= "<td>" . $data['uri'] . "</td>\n";
	$out .= "<td>";
	$out .= "<a href='execute_server_data.php?id=" . $Id . "&type=rdfsproperty&data=" . urlencode($data['uri']) . "'>Execute</a>";
	$out .= "</td>\n";
	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a unknown Object as htmltableline
 * 
 * @param array $data unknown Object as internaltype
 * @return html -table line with data
 */
function print_unknown($data, $Id)
{
	$out .= "<tr>\n";
	$out .= "<td><pre>";
	$out .= print_r($data, true);
	$out .= "</pre></td>\n";
	$out .= "</tr>\n";
	return $out;
} 

/**
 * Prints data of a simple type as htmltableline
 * 
 * @param  $data simple data
 * @return html -table line with data
 */

function print_simple($data)
{
	return "<tr><td>" . highlight_string($data['data'], true) . "</td><tr>";
} 

/**
 * Prints error message as htmltableline
 * 
 * @return html -table line with errormessage
 */
function print_null()
{
	return "<tr><th>Error, result was NULL!</th></tr>\n";
} 

function print_error($errorMessage)
{
	return '<p><span style="color:red;">' . htmlentities($errorMessage) . '</span></p>';
} 

?>