<?
/**
 * Class that represents an OWL ontology node characterising a class description.
 *
 * @package OWLAPI
 * @author Sren Auer <soeren@auer.cx>
 * @copyright Copyright (c) 2004
 * @version $Id: class.php,v 1.31 2006/01/31 19:01:14 soerenauer Exp $
 * @access public
 **/
class OWLClass  extends RDFSClass {
	function listEquivalentClasses() {
		return $this->listPropertyValues($GLOBALS['OWL_equivalentClass'],'Class');
	}
	function listEquivalentClassesInfered() {
		return $this->listPropertyValuesSymmetric($GLOBALS['OWL_equivalentClass'],'Class');
	}
	function listDisjointWith() {
		return $this->listPropertyValues($GLOBALS['OWL_disjointWith']);
	}
	function listIntersectionOf() {
		return $this->model->getList(parray_shift($this->listPropertyValues($GLOBALS['OWL_intersectionOf'])),'Class');
	}
	function listUnionOf() {
		return $this->model->getList(parray_shift($this->listPropertyValues($GLOBALS['OWL_unionOf'])));
	}
	function listComplementOf() {
		return $this->listPropertyValues($GLOBALS['OWL_complementOf']);
	}
	function setEquivalentClasses($values) {
		return $this->setPropertyValues($GLOBALS['OWL_equivalentClass'],$values);
	}
	function setDisjointWith($values) {
		return $this->setPropertyValues($GLOBALS['OWL_disjointWith'],$values);
	}
	function setIntersectionOf($values) {
		$list=$this->model->addList($values,false);
		return $this->setPropertyValues($GLOBALS['OWL_intersectionOf'],$list);
	}
	function setUnionOf($values) {
		if($values==array_keys($this->listUnionOf()))
			return;
		$list=$this->model->addList($values,false);
		return $this->setPropertyValues($GLOBALS['OWL_unionOf'],$list);
	}
	function setComplementOf($values) {
		return $this->setPropertyValues($GLOBALS['OWL_complementOf'],$values);
	}
	function listSubClassesInfered($includeEquivalentClasses=true) {
		$ret=$this->listSubClasses();
		if($includeEquivalentClasses)
			foreach($this->listEquivalentClassesInfered() as $eqiv)
				$ret=array_merge($ret,$eqiv->listSubClassesInfered(false));
		$ret=array_merge($ret,$this->model->listSubClassesInfered($this));
#		foreach($this->model->listTopClasses() as $key=>$topClass)
#			if(method_exists($topClass,'listIntersectionOf') && $inferedSuperClasses=$topClass->listIntersectionOf())
#				foreach($inferedSuperClasses as $inferedSuperClass)
#					if($inferedSuperClass->getURI()==$this->getURI())
#						$ret[$topClass->getLocalName()]=$topClass;
		return $ret;
	}
	function listSuperClassesInfered() {
		$ret=$this->listSuperClasses();
		foreach($this->listIntersectionOf() as $inferedSuperClass)
			if(!isBnode($inferedSuperClass))
				$ret[$inferedSuperClass->getLocalName()]=$inferedSuperClass;
		return $ret;
	}
	/**
	 * Helper function returning the statement stating the cardinality
	 * of a property
	 *
	 * @param RDFSProperty $property
	 * @param string $minmax
	 * @return Statement
	 * @access private
	 **/
	function _getCardinalityStm($property,$minmax) {
		if(!is_a($property,'Resource'))
			$property=new Resource($property);

		$sql="SELECT s4.subject,s4.predicate,s4.object,s4.l_language,s4.l_datatype,s4.subject_is,s4.object_is,s4.object_is,s4.object
				FROM statements s1
				INNER JOIN statements s2 ON(s1.modelID=s2.modelID AND s2.subject=s1.object)
				INNER JOIN statements s3 ON(s1.modelID=s3.modelID AND s3.subject=s2.subject)
				INNER JOIN statements s4 ON(s1.modelID=s4.modelID AND s4.subject=s2.subject)
			WHERE
				s1.subject='".$this->getURI()."' AND s1.predicate='".$GLOBALS['RDFS_subClassOf']->getURI()."'
				AND s2.subject_is='b' AND s2.predicate='".$GLOBALS['RDF_type']->getURI().
					"' AND s2.object='".$GLOBALS['OWL_Restriction']->getURI()."'
				AND s3.predicate='".$GLOBALS['OWL_onProperty']->getURI()."' AND s3.object='".$property->getURI()."'
				AND (s4.predicate='".$GLOBALS['OWL_cardinality']->getURI()."' OR s4.predicate='".$GLOBALS['OWL_'.$minmax.'Cardinality']->getURI()."')
				AND s1.modelID IN (".$this->model->getModelIds().')';
		if($rs=$this->model->dbConn->execute($sql))
			$stmts=$this->model->_convertRecordSetToMemModel($rs);
		if(!empty($stmts->triples[0]))
			return $stmts->triples[0];
		else return;
	}
	/**
	 * Returns the minmimum or maximum cardinality of a property.
	 *
	 * @param RDFSProperty $property
	 * @param string $minmax
	 * @return minmimum or maximum cardinality of a property
	 * @access private
	 **/
	function _getCardinality($property,$minmax) {
		$stm=$this->_getCardinalityStm($property,$minmax);
		if(!empty($stm) && is_a($stm->obj,'Literal'))
			return $stm->obj->label;
	}
	/**
	 * Returns the minimum cardinality of a property.
	 *
	 * @param RDFSProperty $property
	 *
	 * @return int Minimum cardinality of a property.
	 **/
	function getCardinalityMin($property) {
		return $this->_getCardinality($property,'min');
	}
	/**
	 * Returns the maximum cardinality of a property.
	 *
	 * @param RDFSProperty $property
	 *
	 * @return int Maximum cardinality of a property
	 **/
	function getCardinalityMax($property) {
		return $this->_getCardinality($property,'max');
	}
	/**
	 * OWLClass::_getCardinalityInherited()
	 *
	 * @param $property
	 * @param $minmax
	 * @return
	 * @access private
	 **/
	function _getCardinalityInherited($property,$minmax) {
		if(!$ret=$this->_getCardinality($property,$minmax))
			foreach($this->listSuperClasses() as $superclass)
				if($ret=$superclass->_getCardinalityInherited($property,$minmax))
					break;
		return $ret;
	}
	/**
	 * Returns the minimum cardinality of a property or a minimum cardinality
	 * for that property inherited from a superclass.
	 *
	 * @param RDFSProperty $property
	 *
	 * @return int Minimum cardinality of a property.
	 **/
	function getCardinalityMinInherited($property) {
		return $this->_getCardinalityInherited($property,'min');
	}
	/**
	 * Returns the maximum cardinality of a property or a maximum cardinality
	 * for that property inherited from a superclass.
	 *
	 * @param RDFSProperty $property
	 *
	 * @return int Maximum cardinality of a property
	 **/
	function getCardinalityMaxInherited($property) {
		return $this->_getCardinalityInherited($property,'max');
	}
	/**
	 * Sets the cardinality for a property.
	 *
	 * @param RDFSProperty $property
	 * @param string $minmax
	 * @param int $cardinality
	 * @return
	 * @access private
	 **/
	function _setCardinality($property,$minmax,$cardinality) {
		if(!is_a($property,'Resource'))
			$property=new Resource($property);
		$cardinalitystmmin=$this->_getCardinalityStm($property,'min');
		$cardinalitystmmax=$this->_getCardinalityStm($property,'max');
		$cardinalitystm=${'cardinalitystm'.$minmax};
		$restriction=is_a($cardinalitystmmin,'Statement')?$cardinalitystmmin->subj:
			(is_a($cardinalitystmmax,'Statement')?$cardinalitystmmax->subj:false);
		$value=new Literal($cardinality);
		$value->setDatatype('http://www.w3.org/2001/XMLSchema#nonNegativeInteger');
		if(is_a($cardinalitystm,'Statement')) { // restriction exists
			if($cardinalitystm->obj->label==$cardinality)
				return;
			else { // update restriction
				$this->model->remove($cardinalitystm);
				if($cardinality)
					$this->model->add($cardinalitystm->subj,$cardinalitystm->pred,$value);
				else {
					$r=$this->model->resourceF($cardinalitystm->subj->getURI());
					$r->remove();
				}
			}
		} else if($cardinality) { // create new restriction
			if(!$restriction) {
				$restriction=new Blanknode($this->model->getUniqueResourceURI(BNODE_PREFIX));
				$this->model->add($restriction,$GLOBALS['RDF_type'],$GLOBALS['OWL_Restriction']);
				$this->model->add($restriction,$GLOBALS['OWL_onProperty'],$property);
				$this->model->add($this,$GLOBALS['RDFS_subClassOf'],$restriction);
			}
			$this->model->add($restriction,$GLOBALS['OWL_'.$minmax.'Cardinality'],$value);
		}
	}
	/**
	 * Sets the cardinality for a property.
	 *
	 * @param RDFSProperty $property
	 * @param int $cardinality
	 * @return
	 **/
	function setCardinality($property,$cardinality) {
		$this->setCardinalityMin($property,$cardinality);
		$this->setCardinalityMax($property,$cardinality);
	}
	/**
	 * Sets the minimum cardinality for a property.
	 *
	 * @param RDFSProperty $property
	 * @param int $cardinality
	 * @return
	 **/
	function setCardinalityMin($property,$cardinality) {
		$this->_setCardinality($property,'min',$cardinality);
	}
	/**
	 * Sets the maximum cardinality for a property.
	 *
	 * @param RDFSProperty $property
	 * @param int $cardinality
	 * @return
	 **/
	function setCardinalityMax($property,$cardinality) {
		$this->_setCardinality($property,'max',$cardinality);
	}
	/**
	 * OWLClass::_getRestrictionStatement()
	 *
	 * @param $property
	 * @param string $type
	 * @return
	 * @access private
	 **/
	function _getRestrictionStatement($property,$types=array('allValuesFrom','someValuesFrom','hasValue')) {
		if(!is_array($types))
			$types=array($types);
		$sql="SELECT s4.subject,s4.predicate,s4.object,s4.l_language,s4.l_datatype,s4.subject_is,s4.object_is,s4.object_is,s4.object
				FROM statements s1
				INNER JOIN statements s2 ON(s1.modelID=s2.modelID AND s2.subject=s1.object)
				INNER JOIN statements s3 ON(s1.modelID=s3.modelID AND s3.subject=s2.subject)
				INNER JOIN statements s4 ON(s1.modelID=s4.modelID AND s4.subject=s2.subject)
			WHERE
				s1.subject='".$this->getURI()."' AND s1.predicate='".$GLOBALS['RDFS_subClassOf']->getURI()."'
				AND s2.subject_is='b' AND s2.predicate='".$GLOBALS['RDF_type']->getURI()."' AND s2.object='".$GLOBALS['OWL_Restriction']->getURI()."'
				AND s3.predicate='".$GLOBALS['OWL_onProperty']->getURI()."' AND s3.object='".$property->getURI()."'
				AND s4.predicate IN ('http://www.w3.org/2002/07/owl#".join("','http://www.w3.org/2002/07/owl#",$types)."')
				AND s1.modelID IN(".$this->model->getModelIds().')';
		if($rs=$this->model->dbConn->execute($sql))
			$stmts=$this->model->_convertRecordSetToMemModel($rs);
		if(!empty($stmts->triples[0]))
			return $stmts->triples[0];
		else return;
	}
	function getRestriction($property,$type=array('allValuesFrom','someValuesFrom','hasValue')) {
		if($stm=$this->_getRestrictionStatement($property,$type))
			return array($this->getLocalName()=>$stm);
		else
			foreach($this->listSuperClassesInfered() as $superclass)
				if($res=$superclass->getRestriction($property,$type))
					return $res;
		return array();
	}
	function getRestrictionAllValuesFrom($property) {
		return $this->getRestriction($property,'allValuesFrom');
	}
	function getRestrictionSomeValuesFrom($property) {
		return $this->getRestriction($property,'someValuesFrom');
	}
	function getRestrictionHasValue($property) {
		return $this->getRestriction($property,'hasValue');
	}
}
?>