<?
/**
 * RDFSInstance
 *
 * @package RDFSAPI
 * @author Sren Auer <soeren@auer.cx>
 * @copyright Copyright (c) 2004
 * @version $Id: instance.php,v 1.54 2006/02/18 17:49:20 soerenauer Exp $
 * @access public
 **/
class DefaultRDFSInstance extends RDFSResource {
	/**
	 * Returns an array of all properties defined for classes which this instance is instance of.
	 *
	 * @return array Array of all properties.
	 **/
	function listProperties() {
		$prop=array();
		foreach($this->listClasses() as $class) {
			$prop=array_merge($prop,$class->listProperties());
		}
		return array_reverse($prop);
	}
	/**
	 * Returns the value (label) of a property.
	 *
	 * @param mixed $prop Property
	 *
	 * @return value of the property. the data type depends on the given widget
	 **/
	function getPropertyValuePlain($prop) {
		static $cache;
		if(!is_a($prop,'Resource'))
			$prop=$this->model->resourceF($prop);

		if(empty($cache[$this->model->modelURI][$this->getURI()][$prop->getURI()]))
			$cache[$this->model->modelURI][$this->getURI()][$prop->getURI()]=($res=$this->model->findNode($this,$prop,NULL))?$res->getLabel():'';
		return $cache[$this->model->modelURI][$this->getURI()][$prop->getURI()];
	}

	/**
	 * Returns an array of the values (labels) of a property.
	 * TODO Depreciated!!!
	 *
	 * @param mixed $prop Property
	 *
	 * @return array Values of the property.
	 **/
	function listPropertyValuesPlain($prop='') {
		$ret=array();
		foreach($this->model->findNodes($this,$prop,NULL) as $val)
			$ret[]=is_a($val,'Resource')?$val->getLocalName():$val->getLabel();
		return $ret;
	}
	/**
	 * Returns an array of the values (labels) of all properties of the instance.
	 *
	 * @return array Values of the all properties of the instance.
	 **/
	function listAllPropertyValuesPlain() {
		$ret=array();
		foreach($this->listProperties() as $class)
			foreach($class as $property)
				$ret[$property->getLocalName()]=$this->listPropertyValuesPlain($property);
		return $ret;
	}
	/**
	 * Sets a value for a property.
	 *
	 * @param RDFSProperty $prop,
	 * @param $value Value for the property.
	 * @return
	 **/
	function setPropertyValue($prop,$value) {
		if(!is_a($prop,'RDFSProperty'))
			$prop=$this->model->propertyF($prop);
#print_r($prop);
		$range=$prop->getRange();
		if($this->model->findNode($this,$prop,NULL))
			$this->removePropertyValues($prop);
		if(!is_a($value,'Node'))
			$value=method_exists($prop,'isObjectProperty') && $prop->isObjectProperty()?
				$this->model->resourceF($value):
				new RDFSLiteral($value,'',isBnode($range) || !$range?'':$range->getURI());
		$this->model->add($this,$prop,$value);
	}
	/**
	 * Adds a value for a property. RDFSProperties might have
	 * more than one value.
	 *
	 * @param RDFSProperty $prop The property the value should be added to.
	 * @param $value Value for the property
	 * @return
	 **/
	function addPropertyValue($prop,$value) {
		if(!is_a($prop,'RDFSProperty'))
			$prop=$this->model->propertyF($prop);
		$range=$prop->getRange();
		if(!in_array($value,$this->listPropertyValuesPlain($prop)))
			$this->model->add($this,$prop,is_a($value,'Node')?$value:
				($prop->isObjectProperty()?$this->model->resourceF($value):new RDFSLiteral($value,'',$range?$range->getURI():NULL)));
	}
	/**
	 * Removes all property values of the given property.
	 *
	 * @param RDFSProperty $prop
	 * @param mixed $value
	 * @return
	 **/
	function removePropertyValues($prop,$value='') {
		if(!is_a($prop,'RDFSProperty'))
			$prop=new $this->model->property($prop,$this->model);
		if($value && !is_a($value,'Node'))
			$value=$prop->isObjectProperty()?$this->model->resourceF($value):new RDFSLiteral($value);
		foreach($this->model->findNodes($this,$prop,NULL) as $val)
			if(!$value || $value->getLabel()==$val->getLabel()) {
				$this->model->remove($this,$prop,$val);
			}
	}
}
?>