<?php
require_once(RDFAPI_INCLUDE_DIR.'rdql/RdqlParser.php');
require_once(RDFAPI_INCLUDE_DIR.'rdql/RdqlEngine.php');
require_once(RDFAPI_INCLUDE_DIR.'rdql/RdqlDbEngine.php');
require_once(RDFAPI_INCLUDE_DIR.'syntax/RdfParser.php');

/**
CREATE TABLE `models` (
  `modelID` bigint(20) NOT NULL default '0',
  `modelURI` varchar(255) NOT NULL default '',
  `baseURI` varchar(255) default '',
  PRIMARY KEY  (`modelID`),
  UNIQUE KEY `m_modURI_idx` (`modelURI`)
);
CREATE TABLE `namespaces` (
  `modelID` bigint(20) NOT NULL default '0',
  `namespace` varchar(255) NOT NULL default '',
  `prefix` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`modelID`,`namespace`),
  KEY `n_mod_idx` (`modelID`)
);
CREATE TABLE `languages` (
  `id` mediumint(8) unsigned NOT NULL auto_increment,
  `language` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `language` (`language`)
);

CREATE TABLE `literals` (
  `id` mediumint(8) unsigned NOT NULL auto_increment,
  `value` text NOT NULL,
  `language` smallint(5) unsigned NOT NULL default '0',
  `datatype` mediumint(8) unsigned NOT NULL default '0',
  PRIMARY KEY  (`id`),
  KEY `language` (`language`),
  KEY `datatype` (`datatype`),
  KEY `value` (`value`(10)),
  FULLTEXT KEY `value_2` (`value`)
);

CREATE TABLE `ns` (
  `id` smallint(5) unsigned NOT NULL auto_increment,
  `ns` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `ns` (`ns`)
);

CREATE TABLE `resources` (
  `id` mediumint(8) unsigned NOT NULL auto_increment,
  `ns` smallint(5) unsigned NOT NULL default '0',
  `name` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `ns` (`ns`,`name`),
  KEY `name` (`name`)
);

CREATE TABLE `statements` (
  `modelID` smallint(5) unsigned NOT NULL default '0',
  `subject` mediumint(8) unsigned NOT NULL default '0',
  `predicate` mediumint(8) unsigned NOT NULL default '0',
  `object` mediumint(8) unsigned NOT NULL default '0',
  `object_is` enum('r','l') NOT NULL default 'r',
  UNIQUE KEY `stm` (`modelID`,`subject`,`predicate`,`object`,`object_is`),
  KEY `object` (`object`),
  KEY `modelID` (`modelID`),
  KEY `object_is` (`object_is`),
  KEY `subject` (`subject`),
  KEY `predicate` (`predicate`)
)
//*/

class RDFSModel extends DefaultRDFSModel {
	var $tablePrefix='';#n_';
	/**
	 * Constructor
	 * Do not call this directly.
	 * Use the method getModel,getNewModel or putModel of the Class DbStore instead.
	 *
	 * @param   object ADOConnection  &$dbConnection
	 * @param   string   $modelURI
	 * @param   string   $modelID
	 * @param   string   $baseURI
	 * @access	public
	 */
	function RDFSModel(&$store, $modelURI) {
#		$this->dbConn =& $dbConnection;
#		$this->modelURI = $modelURI;
#		$this->modelID = $modelID;
		$this->tablePrefix=$GLOBALS['_POWL']['db']['tablePrefix'];
		$this->tables=array(
			'ns'=>$this->tablePrefix.'ns',
			'literals'=>$this->tablePrefix.'literals',
			'languages'=>$this->tablePrefix.'languages',
			'resources'=>$this->tablePrefix.'resources',
			'statements'=>$this->tablePrefix.'statements',
		);
		$modelVars =& $store->dbConn->execute("SELECT modelURI, modelID, baseURI FROM `".$this->tablePrefix."models` WHERE modelURI='" .$modelURI ."'");
		$this->modelID 	= $modelVars->fields[1];
		$this->baseURI 	= $this->_checkBaseURI($modelVars->fields[2]);
		parent::DefaultRDFSModel(&$store, $modelURI);
#		$this->baseURI = $this->_checkBaseURI($baseURI);
	}
	/**
	* Load a model from a file containing RDF, N3 or N-Triples.
	* This function recognizes the suffix of the filename (.n3 or .rdf) and
	* calls a suitable parser, if no $type is given as string ("rdf" "n3" "nt");
	* If the model is not empty, the contents of the file is added to this DbModel.
	*
	* @param 	string 	$filename
	* @param 	string 	$type
	* @access	public
	*/
	function load($filename, $type = NULL, $stream=false) {
		if ((isset($type)) && ($type =='n3') OR ($type =='nt')) {
			$parser = new N3Parser();
		}elseif ((isset($type)) && ($type =='rdf')) {
			$parser = new RdfParser();
		}else {	
		// create a parser according to the suffix of the filename
		// if there is no suffix assume the file to be XML/RDF
		preg_match("/\.([a-zA-Z0-9_]+)$/", $filename, $suffix);   
		if (isset($suffix[1]) && (strtolower($suffix[1]) == 'n3') OR (strtolower($suffix[1]) == 'nt'))
			$parser = new N3Parser();
		else
			$parser = new RdfParser();
		};
		
		if($stream && is_a($parser,'rdfparser'))
			$parser->generateModel($filename,false,$this);
		else {
			$temp =& $parser->generateModel($filename);
			$this->addModel($temp);
			if ($this->getBaseURI() == NULL)
				$this->setBaseURI($temp->getBaseURI());  
		}
	}
	function add($subject,$predicate='',$object='') {
		$statement=!$object?$subject:$this->_createStatement($subject,$predicate,$object);
		$predicate=$statement->getPredicate();
		$object=$statement->getObject();
		$subject=$statement->getSubject();
		$sql='INSERT INTO '.$this->tables['statements'].' VALUES ('.$this->modelID.','.$this->_getNodeId($subject).','.$this->_getNodeId($predicate).','.$this->_getNodeId($object).',\''.$this->_getNodeType($object).'\')';
		$this->dbConn->execute($sql);
	}
	function countClasses($includeImports=false) {
		$clsql='';
		foreach($this->vocabulary['Class'] as $cl)
			$clsql.=" OR object='".$this->_getNodeId($cl)."'";
		return $this->dbConn->getOne("SELECT COUNT(modelID) FROM ".$this->tables['statements']." s
				INNER JOIN ".$this->tables['resources']." r ON(s.subject=r.id)
				INNER JOIN ".$this->tables['ns']." sns ON(r.ns=sns.id)
			WHERE (modelID=".$this->modelID.($includeImports?$this->importsSQL:'').")
				AND sns.ns!=''	AND predicate='".$this->_getNodeId($GLOBALS['RDF_type'])."' AND (0 $clsql)");
	}
	function _dbId($resource) {
		if(is_string($resource))
			$resource=is_a($GLOBALS[$resource],'resource')?$GLOBALS[$resource]:$this->resourceF($resource);
		return $this->_getNodeId($resource);
	}
	function _getNodeId($node) {
		if(!is_a($node,'Node'))
			$node=new Resource($node);
		else if(is_a($node,'Resource'))
			$node=new Resource($node->getURI());
		if(is_a($node,'Resource')) {
			static $ids;
			if(!empty($ids[$node->getURI()]))
				return $ids[$node->getURI()];
			if(!$id=$this->dbConn->getOne('SELECT id FROM '.$this->tables['resources'].' WHERE ns='.$this->_getNsId($node->getNamespace()).' AND name=\''.$node->getLocalName().'\'')) {
				$this->dbConn->execute('INSERT INTO '.$this->tables['resources'].' VALUES (NULL,\''.$this->_getNsId($node->getNamespace()).'\',\''.$node->getLocalName().'\')');
				$id=$this->dbConn->Insert_ID();
			}
			$ids[$node->getURI()]=$id;
		} else {
			$this->dbConn->execute('INSERT INTO '.$this->tables['literals'].' VALUES (NULL,'.$this->dbConn->Quote($node->getLabel()).','.$this->_getLanguageId($node->getLanguage()).','.$this->_getNodeId($node->getDatatype()).')');
			$id=$this->dbConn->Insert_ID();
		}
		return $id;
	}
	function _getNodeById($id) {
		if($row=$this->dbConn->getOne('SELECT CONCAT(ns.ns,r.name) FROM '.$this->tables['resources'].' r INNER JOIN '.$this->tables['ns'].' ns ON(r.ns=ns.id) WHERE r.id='.$id))
			return $this->resourceF($row);
	}
	function _getNsId($ns) {
		static $ids;
		if(!empty($ids[$ns]))
			return $ids[$ns];
		if(!$ids[$ns]=$this->dbConn->getOne('SELECT id FROM '.$this->tables['ns'].' WHERE ns=\''.$ns.'\'')) {
			$this->dbConn->execute('INSERT INTO '.$this->tables['ns'].' VALUES (NULL,\''.$ns.'\')');
			$ids[$ns]=$this->dbConn->Insert_ID();
		}
		return $ids[$ns];
	}
	function _getLanguageId($ns) {
		static $ids;
		if(!empty($ids[$ns]))
			return $ids[$ns];
		if(!$ids[$ns]=$this->dbConn->getOne('SELECT id FROM '.$this->tables['languages'].' WHERE language=\''.$ns.'\'')) {
			$this->dbConn->execute('INSERT INTO '.$this->tables['languages'].' VALUES (NULL,\''.$ns.'\')');
			$ids[$ns]=$this->dbConn->Insert_ID();
		}
		return $ids[$ns];
	}
	function _getNodeType($node) {
		if(!is_a($node,'Node'))
			$node=new Resource($node);
		return is_a($node,'Resource')?'r':'l';
	}
	/**
	 * Returns array of all named classes not being sub-class of any other class in this model.
	 * 
	 * @return array of RDFSCLass Objects
	 **/
	function listTopClasses() {
		$args=func_get_args();
		if($c=cache('listTopClasses'.$this->modelURI,$args))
			return $c;
		$clsql='';
		foreach($this->vocabulary['Class'] as $cl)
			$clsql.=" OR s1.object='".$this->_getNodeId($cl)."'";
		$sql="SELECT s1.subject,s2.object_is,s2.object,
				".(!strcasecmp($GLOBALS['_POWL']['db']['type'],'SQLite')?'0':"count(distinct s2.object_is)")." cois
		      FROM ".$this->tables['statements']." s1
				INNER JOIN ".$this->tables['resources']." r ON(s1.subject=r.id)
				INNER JOIN ".$this->tables['ns']." sns ON(r.ns=sns.id)
			  	LEFT JOIN ".$this->tables['statements']." s2 ON(s1.subject=s2.subject AND s2.modelID IN (".$this->getModelIds().") AND s2.predicate='".$this->_getNodeId($GLOBALS['RDFS_subClassOf'])."')
		      WHERE sns.ns<>'' AND s1.predicate='".$this->_getNodeId($GLOBALS['RDF_type'])."'
		         AND (0 $clsql)
		         AND s1.modelID IN (".$this->getModelIds().")
			  GROUP BY s1.subject
			  HAVING cois=0 OR (cois=1 AND s2.object_is='b') OR s2.object=".$this->_getNodeId($GLOBALS['RDFS_Resource'])." OR (s2.object_is='r' AND NOT s2.object LIKE '".$this->baseURI."%')
			  ORDER BY s1.subject";
#print_r($sql);
		$topclasses=array();
		foreach($this->dbConn->getCol($sql) as $id) {
			$stm=$this->_getNodeById($id);
			$topclasses[$stm->getLocalName()]=$this->classF($stm->getURI());
		}
#print_r($topclasses);
#echo("\n".timer()."\n");
		return cache('listTopClasses'.$this->modelURI,$args,$topclasses);
 	}
	function listClassLabelLanguages() {
		foreach($this->vocabulary['Class'] as $cl)
			$clsql.=" OR s2.object='".$cl->getURI()."'";
		$sql="SELECT l.language
		      FROM statements s1 INNER JOIN statements s2
		         ON(s1.subject=s2.subject AND s1.modelID=s2.modelID
		            AND s1.predicate='".$this->_dbId($GLOBALS['RDFS_label'])."'
		            AND s2.predicate='".$this->_dbId($GLOBALS['RDF_type'])."'
		            AND (0 ".$clsql."))
			  	LEFT JOIN ".$this->tables['literals']." l ON(s1.object=l.id)
				WHERE s1.modelID IN (".$this->getModelIds().")
		      GROUP BY l.language";
		return $this->dbConn->getCol($sql);
	}
	function query($subject,$predicate,$object,$start=0,$count='') {
		$sql='SELECT CONCAT(sns.ns,s.name) subj,CONCAT(pns.ns,p.name),';
		if(is_null($object))
			$sql.='IF(object_is=\'r\',CONCAT(ons.ns,o.name),l.value),llang.language,CONCAT(lns.ns,ldtype.name)';
		else if(is_a($object,'Resource'))
			$sql.='CONCAT(ons.ns,o.name),\'\',\'\'';
		else if(is_a($object,'Resource'))
			$sql.='l.value,llang.language,CONCAT(lns.ns,ldtype.name)';
		$sql.=',IF(sns.ns=\'\',\'b\',\'r\'),IF(ons.ns=\'\',\'b\',object_is) FROM '.$this->tables['statements'].'
					INNER JOIN '.$this->tables['resources'].' s ON(subject=s.id)
					INNER JOIN '.$this->tables['ns'].' sns ON(s.ns=sns.id)
					INNER JOIN '.$this->tables['resources'].' p ON(predicate=p.id)
					INNER JOIN '.$this->tables['ns'].' pns ON(p.ns=pns.id)';
		if(is_null($object) || is_a($object,'Resource'))
			$sql.=' LEFT JOIN '.$this->tables['resources'].' o ON(object=o.id) LEFT JOIN '.$this->tables['ns'].' ons ON(o.ns=ons.id)';
		if(is_null($object) || is_a($object,'Literal'))
			$sql.=' LEFT JOIN '.$this->tables['literals'].' l ON(object=l.id)
					LEFT JOIN '.$this->tables['languages'].' llang ON(l.language=llang.id)
					LEFT JOIN '.$this->tables['resources'].' ldtype ON(l.datatype=ldtype.id)
					LEFT JOIN '.$this->tables['ns'].' lns ON(ldtype.ns=lns.id)';
#		$sql='SELECT subject,predicate,object,object_is FROM '.$this->tables['statements'];
		$where.=' WHERE modelID IN ('.$this->getModelIds().')';
		if(!is_null($subject))
			$where.=' AND subject='.$this->_getNodeId($subject);
		if(!is_null($predicate))
			$where.=' AND predicate='.$this->_getNodeId($predicate);
		if(!is_null($object))
			$where.=' AND object='.$this->_getNodeId($object);

#print_r((is_a($subject,'resource')?$subject->getURI():$subject)."\n".(is_a($predicate,'resource')?$predicate->getURI():$predicate)."\n".(is_a($object,'resource')?$object->getURI():$object)."\n".$sql.$where.timer()); #exit;

		$recordSet =& $this->dbConn->execute($sql.$where.($count?" LIMIT $start,$count":' ORDER BY subj'));
		if($count)
			$recordSet->_maxRecordCount=$this->dbConn->getOne('SELECT COUNT(*) FROM '.$this->tables['statements'].$where);
		if (!$recordSet)
			echo $this->dbConn->errorMsg();
		else
			return $recordSet;
	}
	function search($search,$where,$compare,$start='',$count='') {
		// dynamic part of the sql statement
		foreach($where as $whe)
			if($compare=='starts')
				$w[]="$whe like '$search%'";
			else if($compare=='contains')
				$w[]="$whe like '%$search%'";
			else if($compare=='regex')
				$w[]="$whe rlike '$search'";
			else {
				if($whe!='object') {
					$r=new $this->resource($search,$this);
					$w[]="$whe='".$this->_dbId($r)."'";
				} else
					$w[]="$whe='$search'";
			}
		// static part of the sql statement
		$sql="SELECT CONCAT(sns.ns,s.name) subj,CONCAT(pns.ns,p.name),IF(object_is='r',CONCAT(ons.ns,o.name),l.value),llang.language,CONCAT(lns.ns,ldtype.name),IF(sns.ns='','b','r'),IF(ons.ns='','b',object_is)
			FROM ".$this->tables['statements'].'
					INNER JOIN '.$this->tables['resources'].' s ON(subject=s.id)
					INNER JOIN '.$this->tables['ns'].' sns ON(s.ns=sns.id)
					INNER JOIN '.$this->tables['resources'].' p ON(predicate=p.id)
					INNER JOIN '.$this->tables['ns'].' pns ON(p.ns=pns.id)
					LEFT JOIN '.$this->tables['resources'].' o ON(object=o.id)
					LEFT JOIN '.$this->tables['ns'].' ons ON(o.ns=ons.id)
					LEFT JOIN '.$this->tables['literals'].' l ON(object=l.id)
					LEFT JOIN '.$this->tables['languages'].' llang ON(l.language=llang.id)
					LEFT JOIN '.$this->tables['resources'].' ldtype ON(l.datatype=ldtype.id)
					LEFT JOIN '.$this->tables['ns'].' lns ON(ldtype.ns=lns.id)';
		$sql.=' WHERE modelID IN ('.$this->getModelIds().") AND (".join(' OR ',$w).')';

		if($count)
			$recordSet=&$this->dbConn->PageExecute($sql,$count,$start/$count+1);
		else
			$recordSet=&$this->dbConn->execute($sql);
	
		if(!$recordSet)
			echo $this->dbConn->errorMsg();
		else
			return $recordSet;
	}
	/**
	 * Perform an RDQL query on this DbModel.
	 * This method returns an associative array of variable bindings.
	 * The values of the query variables can either be RAP's objects (instances of Node)
	 * if $returnNodes set to TRUE, or their string serialization.
	 *
	 * @access	public
	 * @param string $queryString
	 * @param boolean $returnNodes
	 * @return  array   [][?VARNAME] = object Node  (if $returnNodes = TRUE)
	 *      OR  array   [][?VARNAME] = string
	 *
	 */
	function rdqlQuery($queryString, $returnNodes = TRUE) {
	
	  $parser = new RdqlParser();
	  $parsedQuery =& $parser->parseQuery($queryString);
#print_r($parsedQuery);
	  // this method can only query this DbModel
	  // if another model was specified in the from clause throw an error
	  if (isset($parsedQuery['sources'][0]))
	      if($parsedQuery['sources'][0] != $this->modelURI) {
	        $errmsg = RDFAPI_ERROR . '(class: DbModel; method: rdqlQuery):';
	        $errmsg .= ' this method can only query this DbModel';
	        trigger_error($errmsg, E_USER_ERROR);
	      }
	
	  $engine = new RdqlNDbEngine();
	  $engine->tables=$this->tables;
	  $res =& $engine->queryModel($this, $parsedQuery, $returnNodes);
	  
	  return $res;
	}
/**
 * Search for triples and return the first matching statement.
 * NULL input for any parameter will match anything.
 * Return an NULL if nothing is found.
 *
 * @param	object Resource	$subject
 * @param	object Resource	$predicate
 * @param	object Node	$object
 * @return	object Statement
 * @throws  PhpError
 * @throws  SqlError
 * @access	public
 */
 function findFirstMatchingStatement($subject, $predicate, $object) {

   if ((!is_a($subject, 'Resource') && $subject != NULL) ||
	   (!is_a($predicate, 'Resource') && $predicate != NULL) ||
	   (!is_a($object, 'Node') && $object != NULL)) {

       $errmsg = RDFAPI_ERROR . '(class: DbModel; method: find): Parameters must be subclasses of Node or NULL';
       trigger_error($errmsg, E_USER_ERROR);
   }
   
   // static part of the sql statement
   $sql = 'SELECT subject, predicate, object, object_is
           FROM statements
           WHERE modelID = ' .$this->modelID;

   // dynamic part of the sql statement
   $sql .= $this->_createDynSqlPart_SPO($subject, $predicate, $object);

   // execute the query
   $recordSet =& $this->dbConn->selectLimit($sql,1);

   if (!$recordSet)
      echo $this->dbConn->errorMsg();
   else {
      if (!$recordSet->fields)
         return NULL;
      else {
         $memModel = $this->_convertRecordSetToMemModel($recordSet);
         return $memModel->triples[0];
      }
   }
 }
/**
 * Create the dynamic part of an sql statement selecting triples with the
 * given parameters ($subject, $predicate, $object).
 *
 * @param	object Resource	$subject
 * @param	object Resource	$predicate
 * @param	object Node	$object
 * @return  string
 * @access	private
 */
 function _createDynSqlPart_SPO($subject, $predicate, $object) {

   // conditions derived from the parameters passed to the function
   $sql='';
   if ($subject != NULL)
      $sql .= " AND subject='" .$this->_dbId($subject) ."'";
   if ($predicate != NULL)
      $sql .= " AND predicate='" .$this->_dbId($predicate) ."'";
   if ($object != NULL) {
      $object_is = $this->_getNodeFlag($object);
      if (is_a($object, 'Resource'))
         $sql .= " AND object='" .$this->_dbId($object) ."'
                   AND object_is ='" .$object_is ."'";
      else  {
         $quotedLiteral = $this->dbConn->qstr($object->getLabel());
         $sql .= " AND object=" .$quotedLiteral ."
                   AND l_language='" .$object->getLanguage() ."'
                   AND l_datatype='" .$object->getDataType() ."'
                   AND object_is ='" .$object_is ."'";
      }
   }
   return $sql;
 }

}
Class RdqlNDbEngine extends RdqlDbEngine {
	var $tablePrefix='n_';

 /**
 * Perform an RDQL Query on the given DbModel.
 *
 * @param   object  DbModel $dbModel
 * @param   array   &$parsedQuery  (the same format as $this->parsedQuery)
 * @param   boolean $returnNodes
 * @return  array   [][?VARNAME] = object Node  (if $returnNodes = TRUE)
 *      OR  array   [][?VARNAME] = string
 * @access  public
 */
 function & queryModel(&$dbModel, &$parsedQuery, $returnNodes = TRUE) {

   $this->model=&$dbModel;
   $this->parsedQuery = &$parsedQuery;

   $sql = $this->generateSql($dbModel->getModelIds());
print_r($sql);
   $recordSet =& $dbModel->dbConn->execute($sql);
   $queryResult = $this->filterQueryResult($recordSet);

   if ($returnNodes)
      return $this->toNodes($queryResult);
   else
      return $this->toString($queryResult);
 }

	/**
	 * Generate a piece of an sql select statement for a variable.
	 * Look first if the given variable is defined as a pattern object.
	 * (So you can select the node type, literal lang and dtype)
	 * If not found - look for subjects and select node label and type.
	 * If there is no result either go to predicates.
	 * Predicates are always resources therefore select only the node label.
	 *
	 * @param   string $varName
	 * @return  string
	 * @access	private
	 */
	function _generateSql_SelectVar ($varName) {
	
	 static $index = 0;
	
	 $sql_select = '';
	
	 if (array_key_exists($varName, $this->rsIndexes))
	    return NULL;
	
	 foreach ($this->parsedQuery['patterns'] as $n => $pattern)
	   if ($varName == $pattern['object']['value']) {
	
	      // select the object label
	      $sql_select .= " IF(s" .$n .".object_is='l',l" .++$n .".value,r" .$n .".name) as _" .ltrim($varName, "?") ." , ";
	      $this->rsIndexes[$varName]['value'] = $index++;
	      // select the node type
	      $sql_select .= " s" .$n .".object_is , ";
	      $this->rsIndexes[$varName]['nType'] = $index++;
	      // select the object language
	      $sql_select .= " la" .$n .".language , ";
	      $this->rsIndexes[$varName]['l_lang'] = $index++;
	      // select the object dtype
	      $sql_select .= " CONCAT(dns" .$n .".ns,d" .$n .".name) , ";
	      $this->rsIndexes[$varName]['l_dtype'] = $index++;
	
	      return $sql_select;
	   }
	
	 foreach ($this->parsedQuery['patterns'] as $n => $pattern)
	   if ($varName == $pattern['subject']['value']) {
	      // select the object label
	      $sql_select .= " s" .++$n .".subject as _" .ltrim($varName, "?") ." , ";
	      $this->rsIndexes[$varName]['value'] = $index++;
	
	      return $sql_select;
	   }
	
	 foreach ($this->parsedQuery['patterns'] as $n => $pattern)
	   if ($varName == $pattern['predicate']['value']) {
	      // select the object label
	      $sql_select .= " s" .++$n .".predicate as _" .ltrim($varName, "?") ." , ";
	      $this->rsIndexes[$varName]['value'] = $index++;
	
	      return $sql_select;
	   }
	}
	/**
	 * Generate SQL SELECT clause.
	 *
	 * @return  string
	 * @throws  PHPError
	 * @access	private
	 */
	function generateSql_SelectClause() {
	
	  $sql_select = 'SELECT';
	  $index = 0;
	  $this->rsIndexes = array();
	  
	  foreach ($this->parsedQuery['selectVars'] as $var)
	    $sql_select .= $this->_generateSql_SelectVar($var);
	
	  if (isset($this->parsedQuery['filters'])) {
	      foreach ($this->parsedQuery['filters'] as $n => $filter) {
	
	        // variables from numeric expressions
	        foreach ($filter['numExprVars'] as $numVar)
	          $sql_select .= $this->_generateSql_SelectVar($numVar);
	        
	        // variables from regex equality expressions
	        foreach ($filter['regexEqExprs'] as $regexEqExpr)
	          $sql_select .= $this->_generateSql_SelectVar($regexEqExpr['var']);
	          
	        // variables from string equality expressions
	        foreach ($filter['strEqExprs'] as $strEqVar)
	            $sql_select .= $this->_generateSql_SelectVar($strEqVar['var']);
	      }
	  }
	
	  return rtrim($sql_select, " , ");
	}


	/**
	 * Generate SQL FROM clause
	 *
	 * @return  string
	 * @access	private
	 */
	function generateSql_FromClause() {
	
	  $sql_from = ' FROM';
	  foreach ($this->parsedQuery['patterns'] as $n => $v)
	    $sql_from.=' '.$this->tables['statements'].' s'.($n+1).'
			LEFT JOIN '.$this->tables['literals'].' l'.($n+1).' ON ( s'.($n+1).'.object = l'.($n+1).'.id AND s'.($n+1).'.object_is = \'l\' )
				LEFT JOIN '.$this->tables['languages'].' la'.($n+1).' ON ( l'.($n+1).'.language = la'.($n+1).'.id)
				LEFT JOIN '.$this->tables['resources'].' d'.($n+1).' ON ( l'.($n+1).'.datatype = d'.($n+1).'.id)
					LEFT JOIN '.$this->tables['ns'].' dns'.($n+1).' ON ( d'.($n+1).'.ns = dns'.($n+1).'.id)
			LEFT JOIN '.$this->tables['resources'].' r'.($n+1).' ON ( s'.($n+1).'.object = r'.($n+1).'.id AND s'.($n+1).'.object_is = \'r\' )
			 , ';
	    
	  return rtrim($sql_from, ' , ');
	}


	/**
	 * Generate an SQL WHERE clause
	 *
	 * @param   integer $modelID
	 * @return  string
	 * @access	private
 	*/
	function generateSql_WhereClause($modelID) {
	
	  $sql_where = ' WHERE';
	  $count_patterns = count($this->parsedQuery['patterns']);
	  foreach ($this->parsedQuery['patterns'] as $n => $pattern) {
	    $sql_where .= ' s' .($n+1) .'.modelID IN (' .$modelID .') AND';
	    foreach ($pattern as $key => $val_1)
	      if ($val_1['value'] && $val_1['value']{0}=='?') {
	        $sql_tmp = ' s' .($n+1) .'.' .$key .'=';
	        // find internal bindings
	        switch ($key) {
	          case 'subject':
	                   if ($pattern['subject']['value'] == $pattern['predicate']['value'])
	                      $sql_where .= $sql_tmp .'s' .($n+1) .'.predicate AND';
	                   elseif ($pattern['subject']['value'] == $pattern['object']['value'])
	                      $sql_where .= $sql_tmp .'s' .($n+1) .'.object AND';
	                   break;
	          case 'predicate':
	                   if ($pattern['predicate']['value'] == $pattern['object']['value'])
	                      $sql_where .= $sql_tmp .'s' .($n+1) .'.object AND';
	        }
	        // find external bindings
	        for ($i=$n+1; $i<$count_patterns; $i++)
	            foreach ($this->parsedQuery['patterns'][$i] as $key2 => $val_2)
	              if ($val_1['value']==$val_2['value']) {
	                 $sql_where .= $sql_tmp .'s' .($i+1) .'.' .$key2 .' AND';
	                 break 2;
	              }
	      }else {
	         if ($key == 'object' && isset($val_1['is_literal'])) {
	             $sql_where .= ' l' .($n+1) .".value='".$val_1['value']."' AND";
	             $sql_where .= ' s' .($n+1) .".object_is='l' AND";
	             $sql_where .= ' l' .($n+1) .".datatype='" .$val_1['l_dtype'] ."' AND";
	             // Lang tags only differentiate literals in rdf:XMLLiterals and plain literals.
	             // Therefore if a literal is datatyped ignore the language.
	             if ($val_1['l_dtype'] == NULL ||
	                 $val_1['l_dtype'] == 'http://www.w3.org/2001/XMLSchema#string' ||
	                 $val_1['l_dtype'] == 'http://www.w3.org/1999/02/22-rdf-syntax-ns#XMLLiteral')
	                 
	                $sql_where .= ' l' .($n+1) .".language='" .$val_1['l_lang'] ."' AND";
	         } else
	             $sql_where .= ' s' .($n+1) .'.' .$key ."='" .$this->model->_getNodeId($val_1['value'])."' AND";
	      }
	  }
	  return rtrim($sql_where, ' AND');
	}
}
/*
$nmodel=new NModel($_ET['store']->dbConn,'testmodel#',0);

$_ET['store']->dbConn->execute('LOCK TABLES '.$this->tablePrefix.'statements WRITE, '.$this->tablePrefix.'resources WRITE, '.$this->tablePrefix.'literals WRITE, '.$this->tablePrefix.'languages WRITE,'.$this->tablePrefix.'ns WRITE');
$parser = new RdfParser();

ini_set('max_execution_time','6000');

$parser->generateModel('C:\Dokumente und Einstellungen\soeren\Eigene Dateien\standards\nciOncology.owl',false,$nmodel);
$_ET['store']->dbConn->execute('UNLOCK TABLES');

echo('Time: '.timer().'s');

echo('<xmp>Time: '.timer().'s');
$rdql="SELECT * WHERE (?x,<rdfs:SubClassOf>,?y)(?y,<rdfs:SubClassOf>,<http://www.mindswap.org/2003/nciOncology.owl#NCI_Kind>)";
#for($i=0;$i<100;$i++)
#$nmodel->find('http://www.mindswap.org/2003/nciOncology.owl#Systemic_Therapy',NULL,NULL);
print_r(count($nmodel->rdqlQuery($rdql)));
echo("\nTime: ".timer().'s');
#for($i=0;$i<100;$i++)
#$_ET['rdfsmodel']->find('http://www.mindswap.org/2003/nciOncology.owl#Systemic_Therapy',NULL,NULL);
print_r(count($_ET['rdfsmodel']->rdqlQuery($rdql)));
echo("\nTime: ".timer().'s');
*/
?>