<?
/**
 * RDFSStore
 * 
 * @package RDFSAPI
 * @author Sren Auer <soeren@auer.cx>
 * @copyright Copyright (c) 2004
 * @version $Id: store.php,v 1.4 2005/02/09 19:31:54 soerenauer Exp $
 * @access public
 **/

class POWLStore extends DefaultPowlStore {
	function listModels() {
		$models=array();
		foreach($GLOBALS['_POWL']['db']['tables'] as $table)
			if($uri=$this->tableExists($table))
				$models[$uri]=$this->getModel($uri);
		return $models;
	}
	function getModel($modelURI,$importedURIs=array()) {
		if($this->modelExists($modelURI))
			return new OWLModel($this,$modelURI);
		return false;
	}
	/**
	 * POWLStore::getNewModel()
	 * 
	 * @param $modelURI
	 * @param string $baseURI
	 * @param string $type
	 * @return 
	 **/
	function getNewModel($modelURI,$baseURI='',$type='RDFS') {
		if($this->modelExists($modelURI))
			return false;
		$mt=parent::getNewModel($modelURI,$baseURI);
		unset($this->_models[$modelURI]);
		if($type=='OWL')
			$mt->add(new Statement(new $mt->resource($modelURI,$this),$GLOBALS['RDF_type'],$GLOBALS['OWL_Ontology']));
		unset($GLOBALS['_POWL']['cachedata']);
		$m=$this->getModel($modelURI);
		return $m;
	}
	/**
	 * POWLStore::loadModel()
	 * 
	 * @param $modelURI
	 * @param unknown $file
	 * @param boolean $loadImports
	 * @param boolean $stream
	 * @return 
	 **/
	function loadModel($modelURI,$file=NULL,$loadImports=false,$stream=false) {
		static $justLoaded;
		$justLoaded[$modelURI]=true;
		$file=$file?$file:$modelURI;
		if($fp=fopen($file,'rb')) {
			if($this->SysOnt) {
				$head=fread($fp,2000);
				fclose($fp);
				preg_match_all('/xmlns:([^=]+)=[\'"]([^"\']+)[\'"]/im',$head,$matches);
				$i=array_search($modelURI,$matches[2]);
				$name=($i!==false)?$matches[1][$i]:$GLOBALS['_POWL']['SysOnt']->getUniqueResourceURI('Modelinstance');
				$modelInst=$this->SysOnt->addInstance($name,'Model');
				$modelInst->setPropertyValue('modelURI',$modelURI);
				foreach($matches[1] as $key=>$val)
					$modelInst->addPropertyValue('modelXMLNS',trim($val).':'.trim($matches[2][$key]));
			}
		} else
			return false;
		$model=$this->getNewModel($modelURI);
		$model->dontCheckForDuplicatesOnAdd=true;
		$model->logStart('Model created',$modelURI);
		$model->logEnd();
	 	$log=$this->logDisabled;
		$model->logDisabled=true;
	 	$model->load($file,$stream);
		$model=$this->getModel($modelURI);
		if($modelInst) {
			$type=$model->getType();
			$modelInst->setPropertyValue('modelType',$type);
		}
		if($type=='OWL' && $loadImports)
			foreach($model->listImports() as $import)
				if($this->modelExists($import->getURI())) {
					if(!$justLoaded[$import->getURI()])
						trigger_error("Imported model \"".$import->getURI()."\" exists!\n",E_USER_WARNING);
				} else {
					pwlOutput("Loading imported model \"".$import->getURI()."\".\n");
					$imp=$this->loadModel($import->getURI(),$import->getURI(),true,$stream);
				}
					
		$this->logDisabled=$log;
		return $model;
	}
	/**
	 * POWLStore::deleteModel()
	 * 
	 * @param $modelURI
	 * @return 
	 **/
	function deleteModel($modelURI) {
		if($this->SysOnt) {
			if($mi=$this->SysOnt->getClass('Model'))
				if($inst=$mi->findInstance(array('modelURI'=>$modelURI)))
					$inst->remove();
		}
		$m=$this->getModel($modelURI);
		$m->delete();
		cache('modelExists',array($modelURI),false);
	}
	/**
	 * Check if the DbModel with the given modelURI is already stored in the database
	 *
	 * @param   string   $modelURI
	 * @return  boolean
	 * @throws	SqlError
	 * @access	public
	 */
	function modelExists($modelURI) {
		foreach($GLOBALS['_POWL']['db']['tables'] as $table)
			if($this->tableExists($table)==$modelURI)
				return $table;
		return false;
	}
	function tableExists($table) {
		if($modelURI=$this->dbConn->getOne('SELECT TRIM(":" FROM short_value) FROM '.$table.' WHERE slot=10056 AND short_value LIKE ":%"'))
			return $modelURI;
		else if(in_array($table,$this->dbConn->MetaTables()))
			return 'http://www.owl-ontologies.com/unnamed.owl#';
		else
			return false;
	}
}
?>