<?
/**
 * RDFSStore
 *
 * @package RDFSAPI
 * @author Sren Auer <soeren@auer.cx>
 * @copyright Copyright (c) 2004
 * @version $Id: store.php,v 1.21 2006/03/06 02:19:04 soerenauer Exp $
 * @access public
 **/

class DefaultPOWLStore extends DBStore {
	var $SysOnt;
	var $store;
	var $_models;
	/**
	 * Constructor:
	 * Set the database connection with the given parameters.
	 *
	 * @param   string   $dbDriver
	 * @param   string   $host
	 * @param   string   $dbName
	 * @param   string   $user
	 * @param   string   $password
	 * @access	public
	 */
	function DefaultPOWLStore($dbDriver,$host,$dbName,$user,$password,$SysOntURI=false) {
		DBStore::DBStore($dbDriver,$host,$dbName,$user,$password);
		// if 'tablePrefix' is set, attach SQL-rewrite function to store
		if(!empty($GLOBALS['_POWL']['db']['tablePrefix']))
			$this->dbConn->fnExecute='pwlRewriteSQL';
		if($SysOntURI)
			$this->SysOnt=$GLOBALS['_POWL']['SysOnt']=$this->getModel($SysOntURI);
	}
	function listModels($returnAsArray=false) {
		$models=array();
		if($ms=parent::listModels()) {
			if($returnAsArray)
				return $ms;
			foreach($ms as $model)
				if($this->aclCheck('View',$model['modelURI']))
					$models[$model['modelURI']]=$this->getModel($model['modelURI']);
		} else
			return $ms;
		return $models;
	}
#	function modelExists($modelURI) {
#		if(!isset($this->_models[$modelURI]))
#			$this->_models[$modelURI]=parent::modelExists($modelURI);
#		return $this->_models[$modelURI];
#	}
	function getModel($modelURI,$importedURIs=array()) {
		$modelURI=is_numeric($modelURI)?$this->dbConn->getOne('SELECT modelURI FROM models WHERE modelID='.$modelURI):$modelURI;
		ob_start(); // prevent DB error message if tables don't exist
		if(!$this->modelExists($modelURI)) {
			if(rtrim($modelURI,'#/')!=$modelURI && $this->modelExists(rtrim($modelURI,'#/'))) {
				$modelURI=rtrim($modelURI,'#/');
			} else if($this->modelExists($modelURI.'/'))
				$modelURI.='/';
			else if($this->modelExists($modelURI.'#'))
				$modelURI.='#';
			else return false;
		}
		$m=new RDFSModel($this,$modelURI);
		ob_end_clean();
		if($m) {
			if($m->getType()=='OWL') {
				$importedURIs[rtrim($modelURI,'#/')]=rtrim($modelURI,'#/');
				$m=new OWLModel($this,$modelURI);
				foreach($m->listImports() as $import) if(is_a($import,'resource')) {
					if(!in_array(rtrim($import->getURI(),'#/'),$importedURIs) && $imp=$this->getModel($import->getURI(),$importedURIs))
						$m->importsIds=array_merge($m->importsIds,array($imp->modelID=>$imp->modelID),!empty($imp->importsIds)?$imp->importsIds:array());
					$importedURIs[rtrim($import->getURI(),'#/')]=rtrim($import->getURI(),'#/');
				}
				if($m->importsIds)
					$m->importsSQL=' OR modelID='.join(' OR modelID=',$m->importsIds);
			}
			return $m;
		} else
			return false;
	}
	/**
	 * POWLStore::getNewModel()
	 *
	 * @param $modelURI
	 * @param string $baseURI
	 * @param string $type
	 * @return
	 **/
	function getNewModel($modelURI,$baseURI='',$type='RDFS') {
		if($this->modelExists($modelURI))
			return false;
		$mt=parent::getNewModel($modelURI,$baseURI);
		unset($this->_models[$modelURI]);
		if($type=='OWL')
			$mt->add(new Statement(new resource($modelURI),$GLOBALS['RDF_type'],$GLOBALS['OWL_Ontology']));
		unset($GLOBALS['_POWL']['cachedata']);
		$m=$this->getModel($modelURI);
		return $m;
	}
	/**
	 * POWLStore::loadModel()
	 *
	 * @param $modelURI
	 * @param unknown $file
	 * @param boolean $loadImports
	 * @param boolean $stream
	 * @return
	 **/
	function loadModel($modelURI,$file=NULL,$loadImports=false,$stream=false) {
		static $justLoaded;
		$justLoaded[$modelURI]=true;
		$file=$file?$file:$modelURI;
		if($fp=fopen($file,'rb')) {
			if($this->SysOnt) {
				$head=fread($fp,2000);
				fclose($fp);
				preg_match_all('/xmlns:([^=]+)=[\'"]([^"\']+)[\'"]/im',$head,$matches);
				$i=array_search($modelURI,$matches[2]);
				$name=($i!==false)?$matches[1][$i]:$GLOBALS['_POWL']['SysOnt']->getUniqueResourceURI('Modelinstance');
				$modelInst=$this->SysOnt->addInstance($name,'Model');
				$modelInst->setPropertyValue('modelURI',$modelURI);
				foreach($matches[1] as $key=>$val)
					$modelInst->addPropertyValue('modelXMLNS',trim($val).':'.trim($matches[2][$key]));
			}
		} else
			return false;
		$model=$this->getNewModel($modelURI);
		$model->dontCheckForDuplicatesOnAdd=true;
		$model->logStart('Model created',$modelURI);
		$model->logEnd();
	 	$log=$this->logDisabled;
		$model->logDisabled=true;
	 	$model->load($file,$stream);
		$model=$this->getModel($modelURI);
		if($modelInst) {
			$type=$model->getType();
			$modelInst->setPropertyValue('modelType',$type);
		}
		if($type=='OWL' && $loadImports)
			foreach($model->listImports() as $import)
				if($this->modelExists($import->getURI())) {
					if(!$justLoaded[$import->getURI()])
						trigger_error("Imported model \"".$import->getURI()."\" exists!\n",E_USER_WARNING);
				} else {
					pwlOutput("Loading imported model \"".$import->getURI()."\".\n");
					$imp=$this->loadModel($import->getURI(),$import->getURI(),true,$stream);
				}

		$this->logDisabled=$log;
		return $model;
	}
	/**
	 * POWLStore::deleteModel()
	 *
	 * @param $modelURI
	 * @return
	 **/
	function deleteModel($modelURI) {
		if($this->SysOnt)
			if($mi=$this->SysOnt->getClass('Model'))
				if($inst=$mi->findInstance(array('modelURI'=>$modelURI)))
					$inst->remove();
		$m=$this->getModel($modelURI);
		$m->delete();
		cache('modelExists',array($modelURI),false);
	}
	/**
	 * Check if the DbModel with the given modelURI is already stored in the database
	 *
	 * @param   string   $modelURI
	 * @return  boolean
	 * @throws	SqlError
	 * @access	public
	 */
	function modelExists($modelURI) {
		$args=func_get_args();
		$c=cache('modelExists',$args);
		if($c!==NULL)
			return $c;
		return cache('modelExists',$args,parent::modelExists($modelURI));
	}
	function aclCheck($accessType,$model='',$property='',$class='',$instance='') {
		if($_SESSION['PWL']['user']=='Admin' || $GLOBALS['_POWL']['deactivateLogin'])
			return true;
		if(is_a($model,'Model'))
			$model=$model->modelURI;
		if(!$user=$GLOBALS['_POWL']['SysOnt']->getInstance($_SESSION['PWL']['user']))
			return false;
		if(!$model)
			return in_array($accessType,$user->listPropertyValuesPlain('userPrivileges'))?true:false;
		else if($accessType=='View')
			return in_array($model,$user->listPropertyValuesPlain('userModelsView'))||in_array($model,$user->listPropertyValuesPlain('userModelsEdit'))?true:false;
		else if($accessType=='Edit')
			return in_array($model,$user->listPropertyValuesPlain('userModelsEdit'))?true:false;
		else
			return $this->aclCompute($_SESSION['PWL']['user'],$accessType,$model,$property,$class,$instance);
	}
	function aclCompute($user,$accessType,$model,$property='',$class='',$instance='') {
		if(is_a($model,'Model'))
			$model=$model->modelURI;
		else if(is_a($model,'RDFSProperty')) {
			$property=$model;
			$model=$property->model->modelURI;
		} else if(is_a($model,'RDFSClass')) {
			$class=$model;
			$model=$class->model->modelURI;
		} else if(is_a($model,'RDFSInstance')) {
			$instance=$model;
			$model=$instance->model->modelURI;
		}
		if(!$this->SysOnt || !$acl=$this->SysOnt->getClass('ACL'))
			return true;

		$ret=true;
		// starting with true or false
		if($rule=$this->aclGet('',''))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// accessType
		if($rule=$this->aclGet('',$accessType))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// user, accessType
		if($rule=$this->aclGet($user,$accessType))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// model
		if($rule=$this->aclGet($user,$accessType,$model))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// property
		if($property && $rule=$this->aclGet($user,$accessType,$model,$property))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// class
		if($class && $rule=$this->aclGet($user,$accessType,$model,'',$class))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// instance
		if($instance && $rule=$this->aclGet($user,$accessType,$model,'','',$instance))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// property, class
		if($property && $class && $rule=$this->aclGet($user,$accessType,$model,$property,$class))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		// property, class, instance
		if($property && $class && $instance && $rule=$this->aclGet($user,$accessType,$model,$property,$class,$instance))
			$ret=$rule->getPropertyValuePlain('aclAllowDeny')=='Deny'?false:true;
		return $ret;
	}
	function aclGet($user,$accessType,$model='',$property='',$class='',$instance='') {
		$acl=$this->SysOnt->getClass('ACL');
		$ta=array(
			$this->SysOnt->modelURI.'aclModel'=>$model,
			$this->SysOnt->modelURI.'aclProperty'=>$property,
			$this->SysOnt->modelURI.'aclClass'=>$class,
			$this->SysOnt->modelURI.'aclInstance'=>$instance,
			$this->SysOnt->modelURI.'aclAccessType'=>$accessType?$this->SysOnt->modelURI.$accessType:'',
			$this->SysOnt->modelURI.'aclUser'=>$user?$this->SysOnt->modelURI.$user:'');
#print_r($ta);
		if($rules=$acl->findInstancesRecursive($ta)) {
#print_r($rules);
			return array_shift($rules);
		}
		else
			false;
	}
}
?>